/********************
 * Program:  DHT22 humidity-temperature sensor tester
 * Description: print humidity and temperature to serial
 ********************/

#include <DHT.h>

//Constants
#define DHTPIN 6     // what pin on the arduino is the DHT22 data line connected to
#define DHTPWRPIN A1 // we can power with a pin

#define DHTTYPE DHT22   // DHT 22  (AM2302)
//#define DHTTYPE DHT11   // DHT 11
DHT dht(DHTPIN, DHTTYPE); // Initialize DHT sensor for normal 16mhz Arduino

void setup() { // to run once
  Serial.begin(38400); // Initialize the serial port
  Serial.println("DHT22 Humidity - Temperature Sensor");
  Serial.println("RH (%)\t\t Temp (°C)");
#ifdef DHTPWRPIN
  pinMode(DHTPWRPIN, OUTPUT);
  digitalWrite(DHTPWRPIN, HIGH);
  delay(500);
#endif     
  dht.begin();
}

void loop() {
  // Reading temperature or humidity takes about 250 milliseconds!
  // Sensor readings may also be up to 2 seconds 'old' (its a very slow sensor)
  float h = dht.readHumidity();
  float t = dht.readTemperature();
  // Check if any reads failed and exit early (to try again).
  if (isnan(h) || isnan(t)) {
    Serial.println("Failed to read from DHT22 sensor!");
    return;
  }

  Serial.print(h); 
  Serial.print(" \t\t");
  Serial.println(t); 
  // Wait a few seconds between measurements. The DHT22 should not be read at a higher frequency of
  // about once every 2 seconds. So we add a 3 second delay to cover this.
  delay(10000);
}   
